/*
 * au_core.c
 *
 * Aureal Vortex AU88xx audio driver module
 * Core Support Functions
 *
 * Copyright (c) 1999 Aureal Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Some of this code is based on the es1371 driver, which is
 * Copyright (C) 1998  Thomas Sailer (sailer@ife.ee.ethz.ch)
 * 
 * Revision History:
 * 
 * 29-07-2000 Warren Chartier <icebalm@bigfoot.com>
 *	Added ManMowers SMP fixes, unable to test these
 *
 * 19-08-2000 Nikos Kalogridis
 *      Split vortex.c into multiple files for easier maintenance
 *
 * 24-08-2000 Warren Chartier <icebalm@bigfoot.com>
 *	Reformatted Code
 * 
 */

#define __NO_VERSION__
#include "au_vortex.h"

/* wrapper around kernel timer that remembers last queued time */
typedef struct au_timer
{
	struct timer_list *timer;
	unsigned long last_time;
} au_timer_t;

/********************************/
/* Vortex core support		*/
/********************************/

void *alloc_core(struct pci_dev *pcidev, au_device_t *dev)
{
	CMCONFIG config;
	int bar_len = dev->dev_id == DEV_AU8820 ? 0x20000 : 0x40000;

	config.dMemBase[0] = RSRCADDRESS(pcidev, 0);
	config.dMemLength[0] = bar_len;
	config.wIOPortBase[0] = RSRCADDRESS(pcidev, 1);
	config.wIOPortLength[0] = 2;
	config.bIRQRegisters[0] = pcidev->irq;
	config.bIRQAttrib[0] = 0;
	config.BusNumber = pcidev->bus->number;

	return allocCore(&config);
}

void *linux_kmalloc(unsigned int size)
{
	return kmalloc(size, GFP_KERNEL);
}

void linux_kfree(void *p)
{
	if (p) kfree(p);
}

void *linux_ioremap(unsigned long offset, unsigned long size)
{
	PSHOW("ioremap called for offset 0x%lx, size %li bytes\n", offset, size);
	return ioremap(offset, size);
}

void linux_iounmap(void *addr)
{
	iounmap(addr);
}

void linux_udelay(unsigned long usecs)
{
	udelay(usecs);
}

void linux_dprintf(const char *fmt, ...)
{
#ifdef DEBUG_AU
	char buf[256];
	va_list ap;
	//strcpy(buf, KERN_DEBUG DRIVER_NAME ": ");
	va_start(ap, fmt);
	vsprintf(buf, fmt, ap);
	va_end(ap);
	printk(buf);
	printk("\n");
#endif
}

void *linux_memset(void *s, int c, unsigned long n)
{
	return memset(s, c, (size_t)n);
}

void *linux_memcpy(void *s, const void *ct, unsigned long n)
{
	return memcpy(s, ct, (size_t)n);
}

unsigned long linux_virt_to_phys(volatile void *addr)
{
	return virt_to_bus(addr);
}

void *linux_phys_to_virt(unsigned long addr)
{
	return bus_to_virt(addr);
}

void linux_init_timer(unsigned long *timer, unsigned long func, unsigned long context)
{
	struct timer_list *new_timer = (struct timer_list *)kmalloc(sizeof(struct timer_list), GFP_KERNEL);
	au_timer_t *autimer = (au_timer_t *)kmalloc(sizeof(autimer), GFP_KERNEL);

	init_timer(new_timer);
	new_timer->function = (void *)func;
	new_timer->data = context;
	autimer->timer = new_timer;
	autimer->last_time = 0;
	*timer = (unsigned long)autimer;
}

void linux_add_timer(unsigned long timer, unsigned long ms)
{
	au_timer_t *autimer = (au_timer_t *)timer;
	autimer->timer->expires = jiffies + ((ms * HZ) / 1000);
	if (autimer->timer->expires > autimer->last_time) {
		autimer->last_time = autimer->timer->expires;
		add_timer(autimer->timer);
	} else {
		//PWARN("ignoring late timer, expires %d\n", (int)autimer->timer->expires);
	}
}

void linux_del_timer(unsigned long timer)
{
	au_timer_t *autimer = (au_timer_t *)timer;
	del_timer(autimer->timer);
	kfree((void *)autimer->timer);
	kfree((void *)autimer);
}

unsigned long linux_jiffies()
{
	return jiffies;
}
