/* 
 * au_utils.c
 * 
 * Aureal Vortex AU88xx audio driver module
 * Utility Functions
 * 
 * Copyright (c) 1999 Aureal Inc.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Some of this code is based on the es1371 driver, which is
 * Copyright (C) 1998  Thomas Sailer (sailer@ife.ee.ethz.ch)
 * 
 * Revision History:
 * 
 * 12-08-2000 Warren Chartier <icebalm@bigfoot.com>
 *	Added sound_proc_get_chip()
 *
 * 19-08-2000 Nikos Kalogridis
 *      Split vortex.c into multiple files for easier maintenance
 *
 * 24-08-2000 Warren Chartier <icebalm@bigfoot.com>
 *	Reformatted Code
 *
 */

#define __NO_VERSION__
#include "au_vortex.h"

/*
 * Find the device that registered the specified minor number
 */
au_device_t *find_device(int minor_num)
{
	au_device_t *dev = dev_head;
	au_device_t *found_dev = (au_device_t *)NULL;
	int dev_type = DEVICE_TYPE(minor_num);

	while(dev) {
		switch(dev_type) {
			case DEV_AUDIO:
				if(dev->dev_audio == minor_num)
					found_dev = dev;
				break;
			case DEV_MIXER:
				if(dev->dev_mixer == minor_num)
					found_dev = dev;
				break;
			case DEV_MIDI:
				if(dev->dev_midi == minor_num)
					found_dev = dev;
				break;
			case DEV_SUNDSP:
				if(dev->dev_sun == minor_num)
					found_dev = dev;
				break;
			default:
				break;
		}
		if(found_dev)
			break;
		dev = dev->next;
	}
	return found_dev;
}

/*
 * Find the device that has that pcidev in it
 */
au_device_t *find_device_by_pcidev(struct pci_dev *pdev)
{
	au_device_t *next_dev = dev_head;
	au_device_t *dev = dev_head;

	PDEBUG("find_device_by_pcidev\n");
	while(next_dev) {
		dev = next_dev;
		if (dev->pcidev == pdev)
			return dev;
		next_dev = dev->next;
	}
	return NULL;
}
	
int add_device_to_list(au_device_t *dev)
{
	/* link to end of device list */
	PDEBUG("add_device_to_list\n");
	if(!dev_head)
		dev_head = dev_tail = dev;
	else {
		dev_tail->next = dev;
		dev_tail = dev;
	}
	return 0;
}	

int remove_device_from_list(au_device_t *dev)
{
	au_device_t *next_dev = dev_head;
	au_device_t *curr_dev = dev_head;
	au_device_t *previous_dev = dev_head;
	int found = 0;

	PDEBUG("remove_device_from_list\n");
	while(next_dev) {
		curr_dev = next_dev;
		if (curr_dev == dev) {
			found = 1;
			if (previous_dev == dev_head)
				dev_head = curr_dev->next;
			else
				previous_dev->next = curr_dev->next;
			break;
		}
		next_dev = curr_dev->next;
		previous_dev = curr_dev;
	}
	return 0;
}	

loff_t au_llseek(struct file *file, loff_t offset, int origin)
{
	return -ESPIPE;
}

char *sound_proc_get_chip(int dev_id)
{
	switch (dev_id) {
	case DEV_AU8810:
		return "AU8810";
	case DEV_AU8820:
		return "AU8820";
	case DEV_AU8830:
		return "AU8830";
	default:
		return "AU88xx";
	}
}
